package sun.security.jgss.krb5;

import javax.crypto.CipherInputStream;
import javax.crypto.spec.SecretKeySpec;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.Cipher;
import javax.crypto.CipherOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import sun.security.jgss.GSSToken;
import java.io.InputStream;
import sun.security.krb5.internal.crypto.Aes256;
import sun.security.krb5.internal.crypto.Aes128;
import sun.security.krb5.internal.crypto.ArcFourHmac;
import java.security.GeneralSecurityException;
import sun.security.krb5.internal.crypto.Des3;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import org.ietf.jgss.GSSException;
import sun.security.krb5.EncryptionKey;

class CipherHelper
{
    private static final int KG_USAGE_SEAL = 22;
    private static final int KG_USAGE_SIGN = 23;
    private static final int KG_USAGE_SEQ = 24;
    private static final int DES_CHECKSUM_SIZE = 8;
    private static final int DES_IV_SIZE = 8;
    private static final int AES_IV_SIZE = 16;
    private static final int HMAC_CHECKSUM_SIZE = 8;
    private static final int KG_USAGE_SIGN_MS = 15;
    private static final boolean DEBUG;
    private static final byte[] ZERO_IV;
    private static final byte[] ZERO_IV_AES;
    private int etype;
    private int sgnAlg;
    private int sealAlg;
    private byte[] keybytes;
    private int proto;
    
    CipherHelper(EncryptionKey encryptionKey) throws GSSException {
        this.proto = 0;
        this.etype = encryptionKey.getEType();
        this.keybytes = encryptionKey.getBytes();
        switch (this.etype) {
            case 1:
            case 3: {
                this.sgnAlg = 0;
                this.sealAlg = 0;
                break;
            }
            case 16: {
                this.sgnAlg = 1024;
                this.sealAlg = 512;
                break;
            }
            case 23: {
                this.sgnAlg = 4352;
                this.sealAlg = 4096;
                break;
            }
            case 17:
            case 18: {
                this.sgnAlg = -1;
                this.sealAlg = -1;
                this.proto = 1;
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported encryption type: " + this.etype);
            }
        }
    }
    
    int getSgnAlg() {
        return this.sgnAlg;
    }
    
    int getSealAlg() {
        return this.sealAlg;
    }
    
    int getProto() {
        return this.proto;
    }
    
    int getEType() {
        return this.etype;
    }
    
    boolean isArcFour() {
        boolean n = false;
        if (this.etype == 23) {
            n = true;
        }
        return n;
    }
    
    byte[] calculateChecksum(int n, byte[] input, byte[] input2, byte[] digest, int start, int length, int n2) throws GSSException {
        switch (n) {
            case 0: {
                try {
                    MessageDigest instance = MessageDigest.getInstance("MD5");
                    instance.update(input);
                    instance.update(digest, start, length);
                    if (input2 != null) {
                        instance.update(input2);
                    }
                    digest = instance.digest();
                    start = 0;
                    length = digest.length;
                    input = null;
                }
                catch (NoSuchAlgorithmException cause) {
                    GSSException ex = new GSSException(11, -1, "Could not get MD5 Message Digest - " + cause.getMessage());
                    ex.initCause(cause);
                    throw ex;
                }
            }
            case 512: {
                return this.getDesCbcChecksum(this.keybytes, input, digest, start, length);
            }
            case 1024: {
                byte[] input3;
                if (input == null && input2 == null) {
                    input3 = digest;
                }
                else {
                    length = ((input != null) ? input.length : 0) + length + ((input2 != null) ? input2.length : 0);
                    input3 = new byte[length];
                    int length2 = 0;
                    if (input != null) {
                        System.arraycopy(input, 0, input3, 0, input.length);
                        length2 = input.length;
                    }
                    System.arraycopy(digest, start, input3, length2, length);
                    int n3 = length2 + length;
                    if (input2 != null) {
                        System.arraycopy(input2, 0, input3, n3, input2.length);
                    }
                    start = 0;
                }
                try {
                    return Des3.calculateChecksum(this.keybytes, 23, input3, start, length);
                }
                catch (GeneralSecurityException cause2) {
                    GSSException o = new GSSException(11, -1, "Could not use HMAC-SHA1-DES3-KD signing algorithm - " + cause2.getMessage());
                    o.initCause(cause2);
                    throw o;
                }
            }
            case 4352: {
                if (input == null && input2 == null) {
                }
                else {
                    length = ((input != null) ? input.length : 0) + length + ((input2 != null) ? input2.length : 0);
                    digest = new byte[length];
                    int length3 = 0;
                    if (input != null) {
                        System.arraycopy(input, 0, digest, 0, input.length);
                        length3 = input.length;
                    }
                    System.arraycopy(digest, start, digest, length3, length);
                    int n4 = length3 + length;
                    if (input2 != null) {
                        System.arraycopy(input2, 0, digest, n4, input2.length);
                    }
                    start = 0;
                }
                try {
                    int usage = 23;
                    if (n2 == 257) {
                        usage = 15;
                    }
                    byte[] calculateChecksum = ArcFourHmac.calculateChecksum(this.keybytes, usage, digest, start, length);
                    byte[] array = new byte[this.getChecksumLength()];
                    System.arraycopy(calculateChecksum, 0, array, 0, array.length);
                    return array;
                }
                catch (GeneralSecurityException cause3) {
                    GSSException o2 = new GSSException(11, -1, "Could not use HMAC_MD5_ARCFOUR signing algorithm - " + cause3.getMessage());
                    o2.initCause(cause3);
                    throw o2;
                }
            }
            default: {
                throw new GSSException(11, -1, "Unsupported signing algorithm: " + this.sgnAlg);
            }
        }
    }
    
    byte[] calculateChecksum(byte[] array, byte[] array2, int n, int n2, int n3) throws GSSException {
        int n4 = ((array != null) ? array.length : 0) + n2;
        byte[] array3 = new byte[n4];
        System.arraycopy(array2, n, array3, 0, n2);
        if (array != null) {
            System.arraycopy(array, 0, array3, n2, array.length);
        }
        switch (this.etype) {
            case 17: {
                try {
                    return Aes128.calculateChecksum(this.keybytes, n3, array3, 0, n4);
                }
                catch (GeneralSecurityException cause) {
                    GSSException ex = new GSSException(11, -1, "Could not use AES128 signing algorithm - " + cause.getMessage());
                    ex.initCause(cause);
                    throw ex;
                }
            }
            case 18: {
                try {
                    return Aes256.calculateChecksum(this.keybytes, n3, array3, 0, n4);
                }
                catch (GeneralSecurityException cause2) {
                    GSSException ex2 = new GSSException(11, -1, "Could not use AES256 signing algorithm - " + cause2.getMessage());
                    ex2.initCause(cause2);
                    throw ex2;
                }
            }
            default: {
                throw new GSSException(11, -1, "Unsupported encryption type: " + this.etype);
            }
        }
    }
    
    byte[] encryptSeq(byte[] array, byte[] plaintext, int start, int len) throws GSSException {
        switch (this.sgnAlg) {
            case 0:
            case 512: {
                try {
                    return this.getInitializedDes(true, this.keybytes, array).doFinal(plaintext, start, len);
                }
                catch (GeneralSecurityException cause) {
                    GSSException ex = new GSSException(11, -1, "Could not encrypt sequence number using DES - " + cause.getMessage());
                    ex.initCause(cause);
                    throw ex;
                }
            }
            case 1024: {
                if (array.length == 8) {
                }
                else {
                    array = new byte[8];
                    System.arraycopy(array, 0, array, 0, 8);
                }
                try {
                    return Des3.encryptRaw(this.keybytes, 24, array, plaintext, start, len);
                }
                catch (Exception cause2) {
                    GSSException o = new GSSException(11, -1, "Could not encrypt sequence number using DES3-KD - " + cause2.getMessage());
                    o.initCause(cause2);
                    throw o;
                }
            }
            case 4352: {
                if (array.length == 8) {
                }
                else {
                    array = new byte[8];
                    System.arraycopy(array, 0, array, 0, 8);
                }
                try {
                    return ArcFourHmac.encryptSeq(this.keybytes, 24, array, plaintext, start, len);
                }
                catch (Exception cause3) {
                    GSSException o2 = new GSSException(11, -1, "Could not encrypt sequence number using RC4-HMAC - " + cause3.getMessage());
                    o2.initCause(cause3);
                    throw o2;
                }
            }
            default: {
                throw new GSSException(11, -1, "Unsupported signing algorithm: " + this.sgnAlg);
            }
        }
    }
    
    byte[] decryptSeq(byte[] array, byte[] ciphertext, int start, int len) throws GSSException {
        switch (this.sgnAlg) {
            case 0:
            case 512: {
                try {
                    return this.getInitializedDes(false, this.keybytes, array).doFinal(ciphertext, start, len);
                }
                catch (GeneralSecurityException cause) {
                    GSSException ex = new GSSException(11, -1, "Could not decrypt sequence number using DES - " + cause.getMessage());
                    ex.initCause(cause);
                    throw ex;
                }
            }
            case 1024: {
                if (array.length == 8) {
                }
                else {
                    array = new byte[8];
                    System.arraycopy(array, 0, array, 0, 8);
                }
                try {
                    return Des3.decryptRaw(this.keybytes, 24, array, ciphertext, start, len);
                }
                catch (Exception cause2) {
                    GSSException o = new GSSException(11, -1, "Could not decrypt sequence number using DES3-KD - " + cause2.getMessage());
                    o.initCause(cause2);
                    throw o;
                }
            }
            case 4352: {
                if (array.length == 8) {
                }
                else {
                    array = new byte[8];
                    System.arraycopy(array, 0, array, 0, 8);
                }
                try {
                    return ArcFourHmac.decryptSeq(this.keybytes, 24, array, ciphertext, start, len);
                }
                catch (Exception cause3) {
                    GSSException o2 = new GSSException(11, -1, "Could not decrypt sequence number using RC4-HMAC - " + cause3.getMessage());
                    o2.initCause(cause3);
                    throw o2;
                }
            }
            default: {
                throw new GSSException(11, -1, "Unsupported signing algorithm: " + this.sgnAlg);
            }
        }
    }
    
    int getChecksumLength() throws GSSException {
        switch (this.etype) {
            case 1:
            case 3: {
                return 8;
            }
            case 16: {
                return Des3.getChecksumLength();
            }
            case 17: {
                return Aes128.getChecksumLength();
            }
            case 18: {
                return Aes256.getChecksumLength();
            }
            case 23: {
                return 8;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported encryption type: " + this.etype);
            }
        }
    }
    
    void decryptData(WrapToken wrapToken, byte[] array, int n, int n2, byte[] array2, int n3) throws GSSException {
        switch (this.sealAlg) {
            case 0: {
                this.desCbcDecrypt(wrapToken, getDesEncryptionKey(this.keybytes), array, n, n2, array2, n3);
                break;
            }
            case 512: {
                this.des3KdDecrypt(wrapToken, array, n, n2, array2, n3);
                break;
            }
            case 4096: {
                this.arcFourDecrypt(wrapToken, array, n, n2, array2, n3);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported seal algorithm: " + this.sealAlg);
            }
        }
    }
    
    void decryptData(WrapToken_v2 wrapToken_v2, byte[] array, int n, int n2, byte[] array2, int n3, int n4) throws GSSException {
        switch (this.etype) {
            case 17: {
                this.aes128Decrypt(wrapToken_v2, array, n, n2, array2, n3, n4);
                break;
            }
            case 18: {
                this.aes256Decrypt(wrapToken_v2, array, n, n2, array2, n3, n4);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported etype: " + this.etype);
            }
        }
    }
    
    void decryptData(WrapToken wrapToken, InputStream inputStream, int n, byte[] array, int n2) throws GSSException, IOException {
        switch (this.sealAlg) {
            case 0: {
                this.desCbcDecrypt(wrapToken, getDesEncryptionKey(this.keybytes), inputStream, n, array, n2);
                break;
            }
            case 512: {
                byte[] buffer = new byte[n];
                try {
                    GSSToken.readFully(inputStream, buffer, 0, n);
                }
                catch (IOException cause) {
                    GSSException ex = new GSSException(10, -1, "Cannot read complete token");
                    ex.initCause(cause);
                    throw ex;
                }
                this.des3KdDecrypt(wrapToken, buffer, 0, n, array, n2);
                break;
            }
            case 4096: {
                byte[] buffer2 = new byte[n];
                try {
                    GSSToken.readFully(inputStream, buffer2, 0, n);
                }
                catch (IOException cause2) {
                    GSSException ex2 = new GSSException(10, -1, "Cannot read complete token");
                    ex2.initCause(cause2);
                    throw ex2;
                }
                this.arcFourDecrypt(wrapToken, buffer2, 0, n, array, n2);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported seal algorithm: " + this.sealAlg);
            }
        }
    }
    
    void decryptData(WrapToken_v2 wrapToken_v2, InputStream is, int len, byte[] array, int n, int n2) throws GSSException, IOException {
        byte[] buffer = new byte[len];
        try {
            GSSToken.readFully(is, buffer, 0, len);
        }
        catch (IOException cause) {
            GSSException ex = new GSSException(10, -1, "Cannot read complete token");
            ex.initCause(cause);
            throw ex;
        }
        switch (this.etype) {
            case 17: {
                this.aes128Decrypt(wrapToken_v2, buffer, 0, len, array, n, n2);
                break;
            }
            case 18: {
                this.aes256Decrypt(wrapToken_v2, buffer, 0, len, array, n, n2);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported etype: " + this.etype);
            }
        }
    }
    
    void encryptData(WrapToken wrapToken, byte[] b, byte[] b2, int off, int len, byte[] b3, OutputStream os) throws GSSException, IOException {
        switch (this.sealAlg) {
            case 0: {
                CipherOutputStream cipherOutputStream = new CipherOutputStream(os, this.getInitializedDes(true, getDesEncryptionKey(this.keybytes), CipherHelper.ZERO_IV));
                cipherOutputStream.write(b);
                cipherOutputStream.write(b2, off, len);
                cipherOutputStream.write(b3);
                break;
            }
            case 512: {
                os.write(this.des3KdEncrypt(b, b2, off, len, b3));
                break;
            }
            case 4096: {
                os.write(this.arcFourEncrypt(wrapToken, b, b2, off, len, b3));
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported seal algorithm: " + this.sealAlg);
            }
        }
    }
    
    byte[] encryptData(WrapToken_v2 wrapToken_v2, byte[] array, byte[] array2, byte[] array3, int n, int n2, int n3) throws GSSException {
        switch (this.etype) {
            case 17: {
                return this.aes128Encrypt(array, array2, array3, n, n2, n3);
            }
            case 18: {
                return this.aes256Encrypt(array, array2, array3, n, n2, n3);
            }
            default: {
                throw new GSSException(11, -1, "Unsupported etype: " + this.etype);
            }
        }
    }
    
    void encryptData(WrapToken wrapToken, byte[] input, byte[] input2, int inputOffset, int inputLen, byte[] input3, byte[] output, int outputOffset) throws GSSException {
        switch (this.sealAlg) {
            case 0: {
                Cipher initializedDes = this.getInitializedDes(true, getDesEncryptionKey(this.keybytes), CipherHelper.ZERO_IV);
                try {
                    int outputOffset2 = outputOffset + initializedDes.update(input, 0, input.length, output, outputOffset);
                    initializedDes.update(input3, 0, input3.length, output, outputOffset2 + initializedDes.update(input2, inputOffset, inputLen, output, outputOffset2));
                    initializedDes.doFinal();
                    break;
                }
                catch (GeneralSecurityException cause) {
                    GSSException ex = new GSSException(11, -1, "Could not use DES Cipher - " + cause.getMessage());
                    ex.initCause(cause);
                    throw ex;
                }
            }
            case 512: {
                byte[] des3KdEncrypt = this.des3KdEncrypt(input, input2, inputOffset, inputLen, input3);
                System.arraycopy(des3KdEncrypt, 0, output, outputOffset, des3KdEncrypt.length);
                break;
            }
            case 4096: {
                byte[] arcFourEncrypt = this.arcFourEncrypt(wrapToken, input, input2, inputOffset, inputLen, input3);
                System.arraycopy(arcFourEncrypt, 0, output, outputOffset, arcFourEncrypt.length);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported seal algorithm: " + this.sealAlg);
            }
        }
    }
    
    int encryptData(WrapToken_v2 wrapToken_v2, byte[] array, byte[] array2, byte[] array3, int n, int n2, byte[] array4, int n3, int n4) throws GSSException {
        byte[] array5;
        switch (this.etype) {
            case 17: {
                array5 = this.aes128Encrypt(array, array2, array3, n, n2, n4);
                break;
            }
            case 18: {
                array5 = this.aes256Encrypt(array, array2, array3, n, n2, n4);
                break;
            }
            default: {
                throw new GSSException(11, -1, "Unsupported etype: " + this.etype);
            }
        }
        System.arraycopy(array5, 0, array4, n3, array5.length);
        return array5.length;
    }
    
    private byte[] getDesCbcChecksum(byte[] array, byte[] input, byte[] input2, int inputOffset, int n) throws GSSException {
        Cipher initializedDes = this.getInitializedDes(true, array, CipherHelper.ZERO_IV);
        int blockSize = initializedDes.getBlockSize();
        byte[] input3 = new byte[blockSize];
        int n2 = n / blockSize;
        int n3 = n % blockSize;
        if (n3 == 0) {
            --n2;
            System.arraycopy(input2, inputOffset + n2 * blockSize, input3, 0, blockSize);
        }
        else {
            System.arraycopy(input2, inputOffset + n2 * blockSize, input3, 0, n3);
        }
        try {
            byte[] array2 = new byte[Math.max(blockSize, (input == null) ? blockSize : input.length)];
            if (input != null) {
                initializedDes.update(input, 0, input.length, array2, 0);
            }
            for (int i = 0; i < n2; inputOffset += blockSize, ++i) {
                initializedDes.update(input2, inputOffset, blockSize, array2, 0);
            }
            byte[] output = new byte[blockSize];
            initializedDes.update(input3, 0, blockSize, output, 0);
            initializedDes.doFinal();
            return output;
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use DES Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private final Cipher getInitializedDes(boolean b, byte[] key, byte[] iv) throws GSSException {
        try {
            IvParameterSpec params = new IvParameterSpec(iv);
            SecretKeySpec key2 = new SecretKeySpec(key, "DES");
            Cipher instance = Cipher.getInstance("DES/CBC/NoPadding");
            instance.init(b ? 1 : 2, key2, params);
            return instance;
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void desCbcDecrypt(WrapToken wrapToken, byte[] array, byte[] input, int inputOffset, int n, byte[] output, int outputOffset) throws GSSException {
        try {
            Cipher initializedDes = this.getInitializedDes(false, array, CipherHelper.ZERO_IV);
            initializedDes.update(input, inputOffset, 8, wrapToken.confounder);
            inputOffset += 8;
            n -= 8;
            int blockSize = initializedDes.getBlockSize();
            for (int n2 = n / blockSize - 1, i = 0; i < n2; inputOffset += blockSize, outputOffset += blockSize, ++i) {
                initializedDes.update(input, inputOffset, blockSize, output, outputOffset);
            }
            byte[] output2 = new byte[blockSize];
            initializedDes.update(input, inputOffset, blockSize, output2);
            initializedDes.doFinal();
            int n3 = output2[blockSize - 1];
            if (n3 < 1 || n3 > 8) {
                throw new GSSException(10, -1, "Invalid padding on Wrap Token");
            }
            wrapToken.padding = WrapToken.pads[n3];
            System.arraycopy(output2, 0, output, outputOffset, blockSize - n3);
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use DES cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void desCbcDecrypt(WrapToken wrapToken, byte[] array, InputStream inputStream, int n, byte[] b, int off) throws GSSException, IOException {
        Cipher initializedDes = this.getInitializedDes(false, array, CipherHelper.ZERO_IV);
        CipherInputStream cipherInputStream = new CipherInputStream(new CipherHelper.WrapTokenInputStream(inputStream, n), initializedDes);
        n -= cipherInputStream.read(wrapToken.confounder);
        int blockSize = initializedDes.getBlockSize();
        for (int n2 = n / blockSize - 1, i = 0; i < n2; off += blockSize, ++i) {
            cipherInputStream.read(b, off, blockSize);
        }
        byte[] b2 = new byte[blockSize];
        cipherInputStream.read(b2);
        try {
            initializedDes.doFinal();
        }
        catch (GeneralSecurityException cause) {
            GSSException o = new GSSException(11, -1, "Could not use DES cipher - " + cause.getMessage());
            o.initCause(cause);
            throw o;
        }
        int n3 = b2[blockSize - 1];
        if (n3 < 1 || n3 > 8) {
            throw new GSSException(10, -1, "Invalid padding on Wrap Token");
        }
        wrapToken.padding = WrapToken.pads[n3];
        System.arraycopy(b2, 0, b, off, blockSize - n3);
        return;
    }
    
    private static byte[] getDesEncryptionKey(byte[] array) throws GSSException {
        if (array.length > 8) {
            throw new GSSException(11, -100, "Invalid DES Key!");
        }
        byte[] array2 = new byte[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = (byte)(array[i] ^ 0xF0);
        }
        return array2;
    }
    
    private void des3KdDecrypt(WrapToken wrapToken, byte[] ciphertext, int start, int len, byte[] array, int n) throws GSSException {
        byte[] decryptRaw;
        try {
            decryptRaw = Des3.decryptRaw(this.keybytes, 22, CipherHelper.ZERO_IV, ciphertext, start, len);
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use DES3-KD Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        int n2 = decryptRaw[decryptRaw.length - 1];
        if (n2 < 1 || n2 > 8) {
            throw new GSSException(10, -1, "Invalid padding on Wrap Token");
        }
        wrapToken.padding = WrapToken.pads[n2];
        System.arraycopy(decryptRaw, 8, array, n, decryptRaw.length - 8 - n2);
        System.arraycopy(decryptRaw, 0, wrapToken.confounder, 0, 8);
        return;
    }
    
    private byte[] des3KdEncrypt(byte[] array, byte[] array2, int n, int n2, byte[] array3) throws GSSException {
        byte[] plaintext = new byte[array.length + n2 + array3.length];
        System.arraycopy(array, 0, plaintext, 0, array.length);
        System.arraycopy(array2, n, plaintext, array.length, n2);
        System.arraycopy(array3, 0, plaintext, array.length + n2, array3.length);
        try {
            return Des3.encryptRaw(this.keybytes, 22, CipherHelper.ZERO_IV, plaintext, 0, plaintext.length);
        }
        catch (Exception cause) {
            GSSException ex = new GSSException(11, -1, "Could not use DES3-KD Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void arcFourDecrypt(WrapToken wrapToken, byte[] ciphertext, int start, int len, byte[] array, int n) throws GSSException {
        byte[] decryptSeq = this.decryptSeq(wrapToken.getChecksum(), wrapToken.getEncSeqNumber(), 0, 8);
        byte[] decryptRaw;
        try {
            decryptRaw = ArcFourHmac.decryptRaw(this.keybytes, 22, CipherHelper.ZERO_IV, ciphertext, start, len, decryptSeq);
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use ArcFour Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        int n2 = decryptRaw[decryptRaw.length - 1];
        if (n2 < 1) {
            throw new GSSException(10, -1, "Invalid padding on Wrap Token");
        }
        wrapToken.padding = WrapToken.pads[n2];
        System.arraycopy(decryptRaw, 8, array, n, decryptRaw.length - 8 - n2);
        System.arraycopy(decryptRaw, 0, wrapToken.confounder, 0, 8);
        return;
    }
    
    private byte[] arcFourEncrypt(WrapToken wrapToken, byte[] array, byte[] array2, int n, int n2, byte[] array3) throws GSSException {
        byte[] plaintext = new byte[array.length + n2 + array3.length];
        System.arraycopy(array, 0, plaintext, 0, array.length);
        System.arraycopy(array2, n, plaintext, array.length, n2);
        System.arraycopy(array3, 0, plaintext, array.length + n2, array3.length);
        byte[] array4 = new byte[4];
        GSSToken.writeBigEndian(wrapToken.getSequenceNumber(), array4);
        try {
            return ArcFourHmac.encryptRaw(this.keybytes, 22, array4, plaintext, 0, plaintext.length);
        }
        catch (Exception cause) {
            GSSException ex = new GSSException(11, -1, "Could not use ArcFour Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private byte[] aes128Encrypt(byte[] array, byte[] array2, byte[] array3, int n, int n2, int usage) throws GSSException {
        byte[] plaintext = new byte[array.length + n2 + array2.length];
        System.arraycopy(array, 0, plaintext, 0, array.length);
        System.arraycopy(array3, n, plaintext, array.length, n2);
        System.arraycopy(array2, 0, plaintext, array.length + n2, array2.length);
        try {
            return Aes128.encryptRaw(this.keybytes, usage, CipherHelper.ZERO_IV_AES, plaintext, 0, plaintext.length);
        }
        catch (Exception cause) {
            GSSException ex = new GSSException(11, -1, "Could not use AES128 Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void aes128Decrypt(WrapToken_v2 wrapToken_v2, byte[] ciphertext, int start, int len, byte[] array, int n, int usage) throws GSSException {
        byte[] decryptRaw;
        try {
            decryptRaw = Aes128.decryptRaw(this.keybytes, usage, CipherHelper.ZERO_IV_AES, ciphertext, start, len);
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use AES128 Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        System.arraycopy(decryptRaw, 16, array, n, decryptRaw.length - 16 - 16);
    }
    
    private byte[] aes256Encrypt(byte[] array, byte[] array2, byte[] array3, int n, int n2, int usage) throws GSSException {
        byte[] plaintext = new byte[array.length + n2 + array2.length];
        System.arraycopy(array, 0, plaintext, 0, array.length);
        System.arraycopy(array3, n, plaintext, array.length, n2);
        System.arraycopy(array2, 0, plaintext, array.length + n2, array2.length);
        try {
            return Aes256.encryptRaw(this.keybytes, usage, CipherHelper.ZERO_IV_AES, plaintext, 0, plaintext.length);
        }
        catch (Exception cause) {
            GSSException ex = new GSSException(11, -1, "Could not use AES256 Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void aes256Decrypt(WrapToken_v2 wrapToken_v2, byte[] ciphertext, int start, int len, byte[] array, int n, int usage) throws GSSException {
        byte[] decryptRaw;
        try {
            decryptRaw = Aes256.decryptRaw(this.keybytes, usage, CipherHelper.ZERO_IV_AES, ciphertext, start, len);
        }
        catch (GeneralSecurityException cause) {
            GSSException ex = new GSSException(11, -1, "Could not use AES128 Cipher - " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        System.arraycopy(decryptRaw, 16, array, n, decryptRaw.length - 16 - 16);
    }
    
    static {
        DEBUG = Krb5Util.DEBUG;
        ZERO_IV = new byte[8];
        ZERO_IV_AES = new byte[16];
    }
    
    class WrapTokenInputStream extends InputStream
    {
        private InputStream is;
        private int length;
        private int remaining;
        private int temp;
        
        public WrapTokenInputStream(InputStream is, int n) {
            this.is = is;
            this.length = n;
            this.remaining = n;
        }
        
        @Override
        public final int read() throws IOException {
            if (this.remaining == 0) {
                return -1;
            }
            this.temp = this.is.read();
            if (this.temp != -1) {
                this.remaining -= this.temp;
            }
            return this.temp;
        }
        
        @Override
        public final int read(byte[] b) throws IOException {
            if (this.remaining == 0) {
                return -1;
            }
            this.temp = Math.min(this.remaining, b.length);
            this.temp = this.is.read(b, 0, this.temp);
            if (this.temp != -1) {
                this.remaining -= this.temp;
            }
            return this.temp;
        }
        
        @Override
        public final int read(byte[] b, int off, int b2) throws IOException {
            if (this.remaining == 0) {
                return -1;
            }
            this.temp = Math.min(this.remaining, b2);
            this.temp = this.is.read(b, off, this.temp);
            if (this.temp != -1) {
                this.remaining -= this.temp;
            }
            return this.temp;
        }
        
        @Override
        public final long skip(long b) throws IOException {
            if (this.remaining == 0) {
                return 0L;
            }
            this.temp = (int)Math.min((long)this.remaining, b);
            this.temp = (int)this.is.skip((long)this.temp);
            this.remaining -= this.temp;
            return (long)this.temp;
        }
        
        @Override
        public final int available() throws IOException {
            return Math.min(this.remaining, this.is.available());
        }
        
        @Override
        public final void close() throws IOException {
            this.remaining = 0;
        }
    }
}
